# -*- coding: utf-8 -*-
"""
Created on Tue Mar 21 17:23:52 2017

@author: sylhare

"""

from nprime import pyprime as p
from tests import custom_unit_test as ut


def main():

    # Demo of the Primary testing functions
    print(p.is_prime(7))
    print(p.fermat(7))
    print(p.miller_rabin(7))

    # Demo of the Prime generating functions
    print(p.generate_primes(7))
    print(p.find_primes(2, 7, p.fermat))

    # Demo of the custom_test functions
    print(ut.custom_test(p.is_prime))

    # Demo of Graphical Prime functions
    p.sacks_plot()
    p.ulam_plot()


if __name__ == '__main__':
    main()



import unittest

from nprime import pyprime

# First primes that the function should succeed at finding
FIRST_PRIMES = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67]

# First non prime that the function should confirm as not prime
FIRST_NOT_PRIMES = [1, 4, 6, 8, 9, 10, 12, 14, 15, 16, 18, 20, 21, 22, 24, 25, 26, 27, 28]

# Carmichael number often trigger false positive for the fermat algorithm
CARMICHAEL = [561, 1105, 1729, 2465, 2821, 6601, 8911, 10585, 15841, 29341]

# The key is the base, the list is the pseudoprimes of that base
PSEUDO_PRIMES = {2: [2047, 3277, 4033, 4681, 8321],
                 3: [121, 703, 1891, 3281, 8401, 8911],
                 4: [341, 1387, 2047, 3277, 4033, 4371],
                 5: [781, 1541, 5461, 5611, 7813],
                 6: [217, 481, 1111, 1261, 2701],
                 7: [25, 325, 703, 2101, 2353, 4525],
                 8: [9, 65, 481, 511, 1417, 2047],
                 9: [91, 121, 671, 703, 1541, 1729]}


class CustomAssertions():
    """assertTrue and assertFalse method for lists,
    keeping the unittest.TestCase CamelCase format"""

    def assertTrueList(self, list_to_test):
        for n in list_to_test:
            self.assertTrue(n)

    def assertFalseList(self, list_to_test):
        for n in list_to_test:
            self.assertFalse(n)


class TestPyPrime(unittest.TestCase):

    def setUp(self):
        pass

    def test_is_zero_not_prime(self):
        """Is zero correctly determined not to be prime?"""
        self.assertFalse(pyprime.pyprime(0))

    def test_is_two_prime(self):
        """Is two correctly determined to be prime?"""
        self.assertTrue(pyprime.pyprime(2), msg='Two is prime!')

    def test_is_four_not_prime(self):
        """Is four correctly determined not to be prime?"""
        self.assertFalse(pyprime.pyprime(4), msg='Four is not prime!')

    def test_negative_number(self):
        """Is a negative number correctly determined not to be prime?"""
        for index in range(-1, -10, -1):
            self.assertFalse(pyprime.pyprime(index), msg='{} should not be determined to be prime'.format(index))

    def test_first_primes_prime(self):
        for n in FIRST_PRIMES:
            self.assertTrue(pyprime.pyprime(n))

    def test_first_non_primes_not_prime(self):
        for n in FIRST_NOT_PRIMES:
            self.assertFalse(pyprime.pyprime(n))

    def test_pseudoprimes_not_prime(self):
        for _, value in PSEUDO_PRIMES.items():
            for n in value:
                self.assertFalse(pyprime.pyprime(n))

    def test_carmichael_numbers_not_prime(self):
        for n in CARMICHAEL:
            self.assertFalse(pyprime.pyprime(n))

    def test_float_return_error_message(self):
        self.assertRaises(TypeError, pyprime.pyprime, 3.8)

    def test_bool_return_error_message(self):
        self.assertRaises(TypeError, pyprime.pyprime, True)

    def test_string_return_error_message(self):
        self.assertRaises(TypeError, pyprime.pyprime, "three")

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Mon 27 17:43:39 2017

@author: sylhare

"""
import codecs
import io
import os
import time

HERE = os.path.abspath(os.path.dirname(__file__))


# --- FILE --- #
def save(string, name="prime"):
    """
    Write the string into a text file
    The file is called 'name + the time' (format YYYY-mm-dd)

    """
    os.getcwd()

    file = open(str(name) + "_" + str(time.strftime('%Y-%m-%d')) + '.txt', 'w')
    file.write(string)
    file.close()

    return os.path.realpath(file.name)


def read_into_lines_list(path, n=0):
    """
    Take a file and store all of its content into a list
    The n allow to skip the n number of lines

    Return a list of string
    """
    content = []
    with open(path, 'r') as file:
        content = file.readlines()

    return content[n:]


def read_with_codecs(*parts):
    """Return multiple read calls to different readable objects as a single
    string."""
    # intentionally *not* adding an encoding option to open
    return codecs.open(os.path.join(HERE, *parts), 'r').read()


def read_advance(*filenames, **kwargs):
    """

    :param filenames:
    :param kwargs:
    :return:
    """
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


# --- TIME --- #
def timex(func, *param):
    """
    Calculates the time a function takes to run

    Return the function's time
    """
    t = time.time()
    func(*param)
    t = time.time() - t

    return t


def convert(markdown_path):
    """Convert a Markdown file to a reStructuredText file with the pypandoc"""
    try:
        import pypandoc
        output = pypandoc.convert(markdown_path, 'rst')
        # pypandoc.convert(markdown_path, 'rst', outputfile="README.rst") # Create the rst file
    except(IOError, ImportError):
        output = open(markdown_path).read()

    return output



import unittest
from nprime import pyprime as p


class TestFermat(unittest.TestCase):

    __test_function = p.fermat

    def setUp(self):
        pass

    def test_is_zero_not_prime(self):
        """Is zero correctly determined not to be prime?"""
        self.assertFalse(self.__test_function(8))

    def test_is_two_prime(self):
        """Is two correctly determined to be prime?"""
        self.assertTrue(p.pyprime(2), msg='Two is prime!')

    def test_is_four_not_prime(self):
        """Is four correctly determined not to be prime?"""
        self.assertFalse(p.pyprime(4), msg='Four is not prime!')

    def test_negative_number(self):
        """Is a negative number correctly determined not to be prime?"""
        for index in range(-1, -10, -1):
            self.assertFalse(p.pyprime(index), msg='{} should not be determined to be prime'.format(index))

    def test_first_primes_prime(self):
        for n in t.FIRST_PRIMES:
            self.assertTrue(p.pyprime(n))

    def test_first_non_primes_not_prime(self):
        for n in t.FIRST_NOT_PRIMES:
            self.assertFalse(p.pyprime(n))

    def test_pseudoprimes_not_prime(self):
        for base, value in t.PSEUDO_PRIMES.items():
            for n in value:
                self.assertFalse(p.pyprime(n))

    def test_carmichael_numbers_not_prime(self):
        for n in t.CARMICHAEL:
            self.assertFalse(p.pyprime(n))

    def test_float_return_error_message(self):
        self.assertRaises(TypeError, p.pyprime, 3.8)

    def test_bool_return_error_message(self):
        self.assertRaises(TypeError, p.pyprime, True)

    def test_string_return_error_message(self):
        self.assertRaises(TypeError, p.pyprime, "three")

    def tearDown(self):
        pass


if __name__ == '__main__':
    unittest.main()


from setuptools import setup, find_packages
from nprime.toolbox import convert

LONG_DESCRIPTION = convert("README.md")

setup(name='nprime',
      version='0.0.9',
      description='Python library for primes',
      long_description=LONG_DESCRIPTION,
      author='sylhare',
      author_email='sylhare@outlook.com',
      url='https://github.com/Sylhare/nrpime',
      license='GNU General Public License v3.0',
      tests_require=['pytest'],
      install_requires=['matplotlib>=2.1'],
      keywords=['prime',
                'fermat',
                'miller rabin',
                'math'],
      packages=find_packages(),
      platforms='any',
      zip_safe=False,
      test_suite='tests.test_pyprime',
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 3",
          "Environment :: Other Environment",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
          "Operating System :: OS Independent",
          "Topic :: Software Development :: Libraries :: Python Modules", ]
      )



# -*- coding: utf-8 -*-
"""
Created on Fri Jan  6 17:16:46 2017

@author: sylhare

"""

import random
import math
import matplotlib.pyplot as plt


# -- Primarity testing functions -- #
def is_prime(n):
    """
    Check if the number "n" is prime, with n > 1.

    Returns a boolean, True if n is prime.

    """
    for i in range(2, int(pow(n, 0.5)) + 1):
        if n % i == 0:
            return False
    return True


def eratosthenes(n):
    """
    :return: a boolean, True if n is prime
    """

    return False


def fermat(n, t=10):
    """
    Probabilistic algorithm
    Taking "t" randoms "a" and testing the Fermat's theorem on number "n" > 1

    Prime probability is right is 1 - 1/(2^t)
    Returns a boolean: True if n passes the tests.

    """
    for _ in range(0, t):
        a = random.randrange(1, n)
        x = pow(a, n - 1, n)  # (a^(n-1)) modulo n

        if x == 1:
            prime = True  # /!\ Probable prime
        else:
            prime = False
            break

    return prime


def miller_rabin(n, t=10):
    """
    A probabilistic algorithm which determines
    whether a given number (n > 1) is prime or not.
    The miller_rabin tests is repeated t times to get more accurate results.

    Returns a boolean: True if n passes the tests

    """
    if n == 2:
        prime = True  # To normalize and make the algorythm works with 2
    else:
        prime = False  # All other even number will output false

    # Step 1: Have n-1 = 2^s * m (with m odd, and s number of twos factored)
    d = n - 1
    s = 0
    while d % 2 == 0:
        d //= 2  # d equals to quotient of d divided 2
        s += 1  # s > 1 when n is odd

    for k in range(0, t):
        #  Step 2: test (a^d)^2^r ≡ 1 mod n for all r
        a = random.randrange(1, n)
        for _ in range(0, s):
            x = pow(a, d * pow(2, s), n)
            if x == 1 or x == -1:
                prime = True  # Should be true for all a
            else:
                return False  # When not true, it's not prime for sure

    return prime  # /!\ Probable prime


# Prime generating functions #
def generate_primes(upper=0):
    """
    Generate a list of primes from 2 to a set limit
    It reuses the sieve of Eratosthenes

    Returns a list of integer.

    """
    primes = [2]

    for n in range(3, upper + 1):
        k = 0

        # We only check if n is divided by the previous primes
        while primes[k] <= pow(n, 0.5) and n % primes[k] != 0:
            k += 1

        # if a number has no dividers, last prime[k] of loop is over n's square root
        if pow(n, 0.5) < primes[k]:
            primes.append(n)

    return primes


def find_primes(lower, upper, prime_test_function=is_prime):
    """
    Find the number of primes between lower and upper range.
    We should have 1 < lower < upper.

    primeTest determines the function used to tests the primality of the number
    primeTest is by default is_prime() and should return a boolean

    Returns a list of integer.

    """
    if 1 > lower >= upper:
        raise ValueError("We should have 1 < lower < upper")

    primes = []

    for n in range(lower, upper + 1):
        if prime_test_function(n):
            primes.append(n)

    return primes


# Other functions #
def pyprime(n, func=is_prime):
    """
    Function that will encapsulate and tests n before doing function(n)
    It is mostly to encapsulate the primality testing functions
    Basically making sure that:
        n is a number
        n > 1

    """
    # assert will trigger an error if the input is not correct
    if type(n) is int:
        if n > 1:
            return func(n)
        else:
            return False
    else:
        raise TypeError("n should be an integer")


# Graphical Prime functions #
def sacks(upper=1000, prime_test_function=pyprime):
    """
    Generate the sack diagram values up to a set limit (upper)

    primeTest determines the function used to tests the primality of the number
    primeTest is by default is_prime() and should return a boolean

    Returns tho lists:
        1- The none prime polar coordinates: coord
        2- The prime polar coordinates: pricoo

    """
    coord = []  # Normal numbers' polar value
    pricoo = []  # Prime numbers' polar value

    for i in range(0, upper):  # A rotation is made for each perfect square,
        theta = math.sqrt(i) * 2 * math.pi  # i=1 theta= 2pi for a given i, angle=(i*theta)/1
        r = math.sqrt(i)

        if prime_test_function(i):
            pricoo.append((theta, r))
        else:
            coord.append((theta, r))
    return coord, pricoo


# noinspection PyCompatibility
def sacks_plot(upper=10000, prime_test_function=pyprime):
    """
    Render the sacks_plot from the sacks function.
    By default the coord is plot in white and the pricoo in black

    Return a polar plot of the sacks' diagram

    """
    coord, pricoo = sacks(upper, prime_test_function)

    plt.figure()
    ax = plt.subplot(111, projection='polar', facecolor='white')
    plt.title('Sacks\' Diagram', loc='right')
    # noinspection PyCompatibility
    ax.plot(*zip(*coord), "w+", markersize=1)
    ax.plot(*zip(*pricoo), "ko", markersize=2)
    plt.show()


def ulam(upper=1000, edge=4, prime_test_function=pyprime):
    """
    Ulam's spiral aim to represent the primes and none primes in a spiral way

    edge (edge>3) determines the polygone size by the number of edges, 3 triangle, 4 rectangle, 5 Pentagone ...
    For odd number of edge, the spiral gets misaligned

    primeTest determines the function used to tests the primality of the number
    primeTest is by default is_prime() and should return a boolean

    Returns tho lists:
        1- The none prime polar coordinates: coord
        2- The prime polar coordinates: pricoo

    """
    theta = 0  # Keep track of the spiral rotation
    psi = math.radians(360 / edge)  # Angle of the polygone's corner

    turn = 3  # Threshold that indicates to turn at the end of each edge's length
    length = 0  # length of the edge, gets bigger as it spirals
    spiral = 2  # Threshold that indicates when to increase the length of an edge
    spiral_increment = int(edge / 2)  # when the edge length has to go up to spiral

    coord = [(0, 0)]  # Other numbers' coordinates
    pricoo = []  # Primes' coordinates
    x = 0
    y = 0

    for i in range(2, upper):
        if i == spiral:
            length += 1
            spiral = length * spiral_increment + i

        if i == turn:
            theta += psi
            turn = i + length

        x += math.cos(theta)
        y += math.sin(theta)

        if prime_test_function(i):
            pricoo.append((x, y))
        else:
            coord.append((x, y))

    return coord, pricoo


def ulam_plot(upper=10000, edge=4, prime_test_function=pyprime):
    """
    Render the sacks_plot from the ulam function.
    By default the coord is plot in white and the pricoo in black

    Return a polar plot of the ulam's spiral

    """
    coord, pricoo = ulam(upper, edge, prime_test_function)

    plt.figure()
    plt.title('Ulam\'s sprial', loc='right')
    plt.plot(*zip(*coord), 'w+', markersize=1)
    plt.plot(*zip(*pricoo), 'ko', markersize=2)
    plt.grid(True)
    plt.show()
